@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/filedumper.c,v 1.54 2010/09/15 18:19:36 amb Exp $
-
  Memory file dumper.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -28,11 +26,13 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <time.h>
+#include <math.h>
 
 #include "types.h"
 #include "nodes.h"
 #include "segments.h"
 #include "ways.h"
+#include "relations.h"
 
 #include "files.h"
 #include "visualiser.h"
@@ -41,13 +41,15 @@
 
 /* Local functions */
 
-static void print_node(Nodes* nodes,index_t item);
+static void print_node(Nodes *nodes,index_t item);
 static void print_segment(Segments *segments,index_t item);
 static void print_way(Ways *ways,index_t item);
+static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
 
 static void print_head_osm(void);
-static void print_node_osm(Nodes* nodes,index_t item);
+static void print_node_osm(Nodes *nodes,index_t item);
 static void print_segment_osm(Segments *segments,index_t item,Ways *ways);
+static void print_turnrelation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
 static void print_tail_osm(void);
 
 static char *RFC822Date(time_t t);
@@ -64,9 +66,10 @@ int main(int argc,char** argv)
  Nodes    *OSMNodes;
  Segments *OSMSegments;
  Ways     *OSMWays;
+ Relations*OSMRelations;
  int       arg;
  char     *dirname=NULL,*prefix=NULL;
- char     *nodes_filename,*segments_filename,*ways_filename;
+ char     *nodes_filename,*segments_filename,*ways_filename,*relations_filename;
  int       option_statistics=0;
  int       option_visualiser=0,coordcount=0;
  double    latmin=0,latmax=0,lonmin=0,lonmax=0;
@@ -110,6 +113,8 @@ int main(int argc,char** argv)
        ;
     else if(!strncmp(argv[arg],"--way=",6))
        ;
+    else if(!strncmp(argv[arg],"--turn-relation=",16))
+       ;
     else
        print_usage(0,argv[arg],NULL);
    }
@@ -125,6 +130,8 @@ int main(int argc,char** argv)
 
  OSMWays=LoadWayList(ways_filename=FileName(dirname,prefix,"ways.mem"));
 
+ OSMRelations=LoadRelationList(relations_filename=FileName(dirname,prefix,"relations.mem"));
+
  /* Write out the visualiser data */
 
  if(option_visualiser)
@@ -136,21 +143,23 @@ int main(int argc,char** argv)
        print_usage(0,NULL,"The --visualiser option must have --data.\n");
 
     if(!strcmp(option_data,"junctions"))
-       OutputJunctions(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputJunctions(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"super"))
-       OutputSuper(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputSuper(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"oneway"))
-       OutputOneway(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputOneway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
+    else if(!strcmp(option_data,"turns"))
+       OutputTurnRestrictions(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"speed"))
-       OutputSpeedLimits(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputSpeedLimits(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"weight"))
-       OutputWeightLimits(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputWeightLimits(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"height"))
-       OutputHeightLimits(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputHeightLimits(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"width"))
-       OutputWidthLimits(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputWidthLimits(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"length"))
-       OutputLengthLimits(OSMNodes,OSMSegments,OSMWays,latmin,latmax,lonmin,lonmax);
+       OutputLengthLimits(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else
        print_usage(0,option_data,NULL);
    }
@@ -169,19 +178,25 @@ int main(int argc,char** argv)
 
     stat(nodes_filename,&buf);
 
-    printf("'%s%snodes.mem'    - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
+    printf("'%s%snodes.mem'     - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
     printf("%s\n",RFC822Date(buf.st_mtime));
     printf("\n");
 
     stat(segments_filename,&buf);
 
-    printf("'%s%ssegments.mem' - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
+    printf("'%s%ssegments.mem'  - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
     printf("%s\n",RFC822Date(buf.st_mtime));
     printf("\n");
 
     stat(ways_filename,&buf);
 
-    printf("'%s%sways.mem'     - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
+    printf("'%s%sways.mem'      - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
+    printf("%s\n",RFC822Date(buf.st_mtime));
+    printf("\n");
+
+    stat(relations_filename,&buf);
+
+    printf("'%s%srelations.mem' - %9lld Bytes\n",prefix?prefix:"",prefix?"-":"",(long long)buf.st_size);
     printf("%s\n",RFC822Date(buf.st_mtime));
     printf("\n");
 
@@ -191,17 +206,17 @@ int main(int argc,char** argv)
     printf("-----\n");
     printf("\n");
 
-    printf("sizeof(Node) =%9d Bytes\n",sizeof(Node));
-    printf("Number       =%9d\n",OSMNodes->file.number);
-    printf("Number(super)=%9d\n",OSMNodes->file.snumber);
+    printf("sizeof(Node) =%9lu Bytes\n",(unsigned long)sizeof(Node));
+    printf("Number       =%9"Pindex_t"\n",OSMNodes->file.number);
+    printf("Number(super)=%9"Pindex_t"\n",OSMNodes->file.snumber);
     printf("\n");
 
-    printf("Lat bins= %4d\n",OSMNodes->file.latbins);
-    printf("Lon bins= %4d\n",OSMNodes->file.lonbins);
+    printf("Lat bins= %4d\n",(int)OSMNodes->file.latbins);
+    printf("Lon bins= %4d\n",(int)OSMNodes->file.lonbins);
     printf("\n");
 
-    printf("Lat zero=%5d (%8.4f deg)\n",OSMNodes->file.latzero,radians_to_degrees(latlong_to_radians(bin_to_latlong(OSMNodes->file.latzero))));
-    printf("Lon zero=%5d (%8.4f deg)\n",OSMNodes->file.lonzero,radians_to_degrees(latlong_to_radians(bin_to_latlong(OSMNodes->file.lonzero))));
+    printf("Lat zero=%5d (%8.4f deg)\n",(int)OSMNodes->file.latzero,radians_to_degrees(latlong_to_radians(bin_to_latlong(OSMNodes->file.latzero))));
+    printf("Lon zero=%5d (%8.4f deg)\n",(int)OSMNodes->file.lonzero,radians_to_degrees(latlong_to_radians(bin_to_latlong(OSMNodes->file.lonzero))));
 
     /* Examine the segments */
 
@@ -210,10 +225,10 @@ int main(int argc,char** argv)
     printf("--------\n");
     printf("\n");
 
-    printf("sizeof(Segment)=%9d Bytes\n",sizeof(Segment));
-    printf("Number(total)  =%9d\n",OSMSegments->file.number);
-    printf("Number(super)  =%9d\n",OSMSegments->file.snumber);
-    printf("Number(normal) =%9d\n",OSMSegments->file.nnumber);
+    printf("sizeof(Segment)=%9lu Bytes\n",(unsigned long)sizeof(Segment));
+    printf("Number(total)  =%9"Pindex_t"\n",OSMSegments->file.number);
+    printf("Number(super)  =%9"Pindex_t"\n",OSMSegments->file.snumber);
+    printf("Number(normal) =%9"Pindex_t"\n",OSMSegments->file.nnumber);
 
     /* Examine the ways */
 
@@ -222,19 +237,31 @@ int main(int argc,char** argv)
     printf("----\n");
     printf("\n");
 
-    printf("sizeof(Way)      =%9d Bytes\n",sizeof(Way));
-    printf("Number(compacted)=%9d\n",OSMWays->file.number);
-    printf("Number(original) =%9d\n",OSMWays->file.onumber);
+    printf("sizeof(Way)      =%9lu Bytes\n",(unsigned long)sizeof(Way));
+    printf("Number(compacted)=%9"Pindex_t"\n",OSMWays->file.number);
+    printf("Number(original) =%9"Pindex_t"\n",OSMWays->file.onumber);
     printf("\n");
 
-    printf("Total names =%9ld Bytes\n",(long)buf.st_size-sizeof(Ways)-OSMWays->file.number*sizeof(Way));
+    stat(ways_filename,&buf);
+    printf("Total names =%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)sizeof(Ways)-(unsigned long)OSMWays->file.number*(unsigned long)sizeof(Way));
     printf("\n");
 
+    printf("Included highways  : %s\n",HighwaysNameList(OSMWays->file.highways));
     printf("Included transports: %s\n",AllowedNameList(OSMWays->file.allow));
     printf("Included properties: %s\n",PropertiesNameList(OSMWays->file.props));
+
+    /* Examine the relations */
+
+    printf("\n");
+    printf("Relations\n");
+    printf("---------\n");
+    printf("\n");
+
+    printf("sizeof(TurnRelation)=%9lu Bytes\n",(unsigned long)sizeof(TurnRelation));
+    printf("Number              =%9"Pindex_t"\n",OSMRelations->file.trnumber);
    }
 
- /* Print out internal data */
+ /* Print out internal data (in plain text format) */
 
  if(option_dump)
    {
@@ -253,7 +280,7 @@ int main(int argc,char** argv)
           if(item>=0 && item<OSMNodes->file.number)
              print_node(OSMNodes,item);
           else
-             printf("Invalid node number; minimum=0, maximum=%d.\n",OSMNodes->file.number-1);
+             printf("Invalid node number; minimum=0, maximum=%"Pindex_t".\n",OSMNodes->file.number-1);
          }
        else if(!strcmp(argv[arg],"--segment=all"))
          {
@@ -267,7 +294,7 @@ int main(int argc,char** argv)
           if(item>=0 && item<OSMSegments->file.number)
              print_segment(OSMSegments,item);
           else
-             printf("Invalid segment number; minimum=0, maximum=%d.\n",OSMSegments->file.number-1);
+             printf("Invalid segment number; minimum=0, maximum=%"Pindex_t".\n",OSMSegments->file.number-1);
          }
        else if(!strcmp(argv[arg],"--way=all"))
          {
@@ -281,11 +308,25 @@ int main(int argc,char** argv)
           if(item>=0 && item<OSMWays->file.number)
              print_way(OSMWays,item);
           else
-             printf("Invalid way number; minimum=0, maximum=%d.\n",OSMWays->file.number-1);
+             printf("Invalid way number; minimum=0, maximum=%"Pindex_t".\n",OSMWays->file.number-1);
+         }
+       else if(!strcmp(argv[arg],"--turn-relation=all"))
+         {
+          for(item=0;item<OSMRelations->file.trnumber;item++)
+             print_turnrelation(OSMRelations,item,OSMSegments,OSMNodes);
+         }
+       else if(!strncmp(argv[arg],"--turn-relation=",16))
+         {
+          item=atoi(&argv[arg][16]);
+
+          if(item>=0 && item<OSMRelations->file.trnumber)
+             print_turnrelation(OSMRelations,item,OSMSegments,OSMNodes);
+          else
+             printf("Invalid turn relation number; minimum=0, maximum=%"Pindex_t".\n",OSMRelations->file.trnumber-1);
          }
    }
 
- /* Print out internal data in XML format */
+ /* Print out internal data (in OSM XML format) */
 
  if(option_dump_osm)
    {
@@ -296,19 +337,24 @@ int main(int argc,char** argv)
 
     if(coordcount)
       {
-       int32_t latminbin=latlong_to_bin(radians_to_latlong(latmin))-OSMNodes->file.latzero;
-       int32_t latmaxbin=latlong_to_bin(radians_to_latlong(latmax))-OSMNodes->file.latzero;
-       int32_t lonminbin=latlong_to_bin(radians_to_latlong(lonmin))-OSMNodes->file.lonzero;
-       int32_t lonmaxbin=latlong_to_bin(radians_to_latlong(lonmax))-OSMNodes->file.lonzero;
-       int latb,lonb,llbin;
+       ll_bin_t latminbin=latlong_to_bin(radians_to_latlong(latmin))-OSMNodes->file.latzero;
+       ll_bin_t latmaxbin=latlong_to_bin(radians_to_latlong(latmax))-OSMNodes->file.latzero;
+       ll_bin_t lonminbin=latlong_to_bin(radians_to_latlong(lonmin))-OSMNodes->file.lonzero;
+       ll_bin_t lonmaxbin=latlong_to_bin(radians_to_latlong(lonmax))-OSMNodes->file.lonzero;
+       ll_bin_t latb,lonb;
        index_t item,index1,index2;
 
+       if(latminbin<0)                      latminbin=0;
+       if(latmaxbin>OSMNodes->file.latbins) latmaxbin=OSMNodes->file.latbins-1;
+       if(lonminbin<0)                      lonminbin=0;
+       if(lonmaxbin>OSMNodes->file.lonbins) lonmaxbin=OSMNodes->file.lonbins-1;
+
        /* Loop through all of the nodes. */
 
        for(latb=latminbin;latb<=latmaxbin;latb++)
           for(lonb=lonminbin;lonb<=lonmaxbin;lonb++)
             {
-             llbin=lonb*OSMNodes->file.latbins+latb;
+             ll_bin2_t llbin=lonb*OSMNodes->file.latbins+latb;
 
              if(llbin<0 || llbin>(OSMNodes->file.latbins*OSMNodes->file.lonbins))
                 continue;
@@ -328,7 +374,7 @@ int main(int argc,char** argv)
 
                    print_node_osm(OSMNodes,item);
 
-                   segment=FirstSegment(OSMSegments,OSMNodes,item);
+                   segment=FirstSegment(OSMSegments,node,1);
 
                    while(segment)
                      {
@@ -338,6 +384,18 @@ int main(int argc,char** argv)
 
                       segment=NextSegment(OSMSegments,segment,item);
                      }
+
+                   if(IsTurnRestrictedNode(node))
+                     {
+                      index_t relindex=FindFirstTurnRelation1(OSMRelations,item);
+
+                      while(relindex!=NO_RELATION)
+                        {
+                         print_turnrelation_osm(OSMRelations,relindex,OSMSegments,OSMNodes);
+
+                         relindex=FindNextTurnRelation1(OSMRelations,relindex);
+                        }
+                     }
                   }
                }
             }
@@ -352,6 +410,9 @@ int main(int argc,char** argv)
        for(item=0;item<OSMSegments->file.number;item++)
           if(!option_no_super || IsNormalSegment(LookupSegment(OSMSegments,item,1)))
              print_segment_osm(OSMSegments,item,OSMWays);
+
+       for(item=0;item<OSMRelations->file.trnumber;item++)
+          print_turnrelation_osm(OSMRelations,item,OSMSegments,OSMNodes);
       }
 
     print_tail_osm();
@@ -362,31 +423,31 @@ int main(int argc,char** argv)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the contents of a node from the routing database.
+  Print out the contents of a node from the routing database (as plain text).
 
   Nodes *nodes The set of nodes to use.
 
   index_t item The node index to print.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_node(Nodes* nodes,index_t item)
+static void print_node(Nodes *nodes,index_t item)
 {
  Node *node=LookupNode(nodes,item,1);
  double latitude,longitude;
 
  GetLatLong(nodes,item,&latitude,&longitude);
 
- printf("Node %d\n",item);
- printf("  firstseg=%d\n",node->firstseg);
+ printf("Node %"Pindex_t"\n",item);
+ printf("  firstseg=%"Pindex_t"\n",node->firstseg);
  printf("  latoffset=%d lonoffset=%d (latitude=%.6f longitude=%.6f)\n",node->latoffset,node->lonoffset,radians_to_degrees(latitude),radians_to_degrees(longitude));
  printf("  allow=%02x (%s)\n",node->allow,AllowedNameList(node->allow));
- if(IsSuperNode(nodes,item))
+ if(IsSuperNode(node))
     printf("  Super-Node\n");
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the contents of a segment from the routing database.
+  Print out the contents of a segment from the routing database (as plain text).
 
   Segments *segments The set of segments to use.
 
@@ -397,10 +458,10 @@ static void print_segment(Segments *segments,index_t item)
 {
  Segment *segment=LookupSegment(segments,item,1);
 
- printf("Segment %d\n",item);
- printf("  node1=%d node2=%d\n",segment->node1,segment->node2);
- printf("  next2=%d\n",segment->next2);
- printf("  way=%d\n",segment->way);
+ printf("Segment %"Pindex_t"\n",item);
+ printf("  node1=%"Pindex_t" node2=%"Pindex_t"\n",segment->node1,segment->node2);
+ printf("  next2=%"Pindex_t"\n",segment->next2);
+ printf("  way=%"Pindex_t"\n",segment->way);
  printf("  distance=%d (%.3f km)\n",DISTANCE(segment->distance),distance_to_km(DISTANCE(segment->distance)));
  if(IsSuperSegment(segment) && IsNormalSegment(segment))
     printf("  Super-Segment AND normal Segment\n");
@@ -414,7 +475,7 @@ static void print_segment(Segments *segments,index_t item)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the contents of a way from the routing database.
+  Print out the contents of a way from the routing database (as plain text).
 
   Ways *ways The set of ways to use.
 
@@ -424,11 +485,12 @@ static void print_segment(Segments *segments,index_t item)
 static void print_way(Ways *ways,index_t item)
 {
  Way *way=LookupWay(ways,item,1);
+ char *name=WayName(ways,way);
 
- printf("Way %d\n",item);
- if(*WayName(ways,way))
-    printf("  name=%s\n",WayName(ways,way));
- printf("  type=%02x (%s%s%s)\n",way->type,HighwayName(HIGHWAY(way->type)),way->type&Way_OneWay?",One-Way":"",way->type&Way_Roundabout?",Roundabout":"");
+ printf("Way %"Pindex_t"\n",item);
+ if(*name)
+    printf("  name=%s\n",name);
+ printf("  type=%02x (%s%s)\n",way->type,HighwayName(HIGHWAY(way->type)),way->type&Way_OneWay?",One-Way":"");
  printf("  allow=%02x (%s)\n",way->allow,AllowedNameList(way->allow));
  if(way->props)
     printf("  props=%02x (%s)\n",way->props,PropertiesNameList(way->props));
@@ -445,6 +507,57 @@ static void print_way(Ways *ways,index_t item)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Print out the contents of a turn relation from the routing database (as plain text).
+
+  Relations *relations The set of relations to use.
+
+  index_t item The turn relation index to print.
+
+  Segments *segments The set of segments to use.
+
+  Nodes *nodes The set of nodes to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
+{
+ Segment *segment;
+ TurnRelation *relation=LookupTurnRelation(relations,item,1);
+ Node *node=LookupNode(nodes,relation->via,1);
+ index_t from_way=NO_WAY,to_way=NO_WAY;
+ index_t from_node=NO_NODE,to_node=NO_NODE;
+
+ segment=FirstSegment(segments,node,1);
+
+ do
+   {
+    index_t seg=IndexSegment(segments,segment);
+
+    if(seg==relation->from)
+      {
+       from_node=OtherNode(segment,relation->from);
+       from_way=segment->way;
+      }
+
+    if(seg==relation->to)
+      {
+       to_node=OtherNode(segment,relation->to);
+       to_way=segment->way;
+      }
+
+    segment=NextSegment(segments,segment,relation->via);
+   }
+ while(segment);
+
+ printf("Relation %"Pindex_t"\n",item);
+ printf("  from=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relation->from,from_way,from_node);
+ printf("  via=%"Pindex_t" (node)\n",relation->via);
+ printf("  to=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relation->to,to_way,to_node);
+ if(relation->except)
+    printf("  except=%02x (%s)\n",relation->except,AllowedNameList(relation->except));
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Print out a header in OSM XML format.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -452,45 +565,55 @@ static void print_way(Ways *ways,index_t item)
 static void print_head_osm(void)
 {
  printf("<?xml version='1.0' encoding='UTF-8'?>\n");
- printf("<osm version='0.6' generator='JOSM'>\n");
+ printf("<osm version='0.6' generator='Routino'>\n");
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the contents of a node from the routing database in OSM XML format.
+  Print out the contents of a node from the routing database (in OSM XML format).
 
   Nodes *nodes The set of nodes to use.
 
   index_t item The node index to print.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_node_osm(Nodes* nodes,index_t item)
+static void print_node_osm(Nodes *nodes,index_t item)
 {
  Node *node=LookupNode(nodes,item,1);
  double latitude,longitude;
+ int i;
 
  GetLatLong(nodes,item,&latitude,&longitude);
 
- if(IsSuperNode(nodes,item))
+ if(node->allow==Transports_ALL && node->flags==0)
+    printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1' />\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
+ else
    {
-    int i;
-
     printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1'>\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
-    printf("    <tag k='routino:super' v='yes' />\n");
+
+    if(node->flags & NODE_SUPER)
+       printf("    <tag k='routino:super' v='yes' />\n");
+
+    if(node->flags & NODE_UTURN)
+       printf("    <tag k='routino:uturn' v='yes' />\n");
+
+    if(node->flags & NODE_MINIRNDBT)
+       printf("    <tag k='highway' v='mini_roundabout' />\n");
+
+    if(node->flags & NODE_TURNRSTRCT)
+       printf("    <tag k='routino:turnrestriction' v='yes' />\n");
 
     for(i=1;i<Transport_Count;i++)
-       if(!(node->allow & ALLOWED(i)))
+       if(!(node->allow & TRANSPORTS(i)))
           printf("    <tag k='%s' v='no' />\n",TransportName(i));
 
     printf("  </node>\n");
    }
- else
-    printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1' />\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the contents of a segment from the routing database as a way in OSM XML format.
+  Print out the contents of a segment from the routing database (as a way in OSM XML format).
 
   Segments *segments The set of segments to use.
 
@@ -503,6 +626,7 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
 {
  Segment *segment=LookupSegment(segments,item,1);
  Way *way=LookupWay(ways,segment->way,1);
+ char *name=WayName(ways,way);
  int i;
 
  printf("  <way id='%lu' version='1'>\n",(unsigned long)item+1);
@@ -523,18 +647,18 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
  if(IsNormalSegment(segment))
     printf("    <tag k='routino:normal' v='yes' />\n");
 
+ printf("    <tag k='routino:distance' v='%.3f' />\n",distance_to_km(DISTANCE(segment->distance)));
+
  if(way->type & Way_OneWay)
     printf("    <tag k='oneway' v='yes' />\n");
- if(way->type & Way_Roundabout)
-    printf("    <tag k='junction' v='roundabout' />\n");
 
  printf("    <tag k='highway' v='%s' />\n",HighwayName(HIGHWAY(way->type)));
 
- if(IsNormalSegment(segment) && *WayName(ways,way))
-    printf("    <tag k='name' v='%s' />\n",ParseXML_Encode_Safe_XML(WayName(ways,way)));
+ if(IsNormalSegment(segment) && *name)
+    printf("    <tag k='name' v='%s' />\n",ParseXML_Encode_Safe_XML(name));
 
  for(i=1;i<Transport_Count;i++)
-    if(way->allow & ALLOWED(i))
+    if(way->allow & TRANSPORTS(i))
        printf("    <tag k='%s' v='yes' />\n",TransportName(i));
 
  for(i=1;i<Property_Count;i++)
@@ -557,6 +681,53 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Print out the contents of a turn relation from the routing database (in OSM XML format).
+
+  Relations *relations The set of relations to use.
+
+  index_t item The relation index to print.
+
+  Segments *segments The set of segments to use.
+
+  Nodes *nodes The set of nodes to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void print_turnrelation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
+{
+ TurnRelation *relation=LookupTurnRelation(relations,item,1);
+
+ Segment *segment_from=LookupSegment(segments,relation->from,1);
+ Segment *segment_to  =LookupSegment(segments,relation->to  ,2);
+
+ double angle=TurnAngle(nodes,segment_from,segment_to,relation->via);
+
+ char *restriction;
+
+ if(angle>150 || angle<-150)
+    restriction="no_u_turn";
+ else if(angle>30)
+    restriction="no_right_turn";
+ else if(angle<-30)
+    restriction="no_left_turn";
+ else
+    restriction="no_straight_on";
+
+ printf("  <relation id='%lu' version='1'>\n",(unsigned long)item+1);
+ printf("    <tag k='type' v='restriction' />\n");
+ printf("    <tag k='restriction' v='%s'/>\n",restriction);
+
+ if(relation->except)
+    printf("    <tag k='except' v='%s' />\n",AllowedNameList(relation->except));
+
+ printf("    <member type='way' ref='%lu' role='from' />\n",(unsigned long)relation->from+1);
+ printf("    <member type='node' ref='%lu' role='via' />\n",(unsigned long)relation->via+1);
+ printf("    <member type='way' ref='%lu' role='to' />\n",(unsigned long)relation->to+1);
+
+ printf("  </relation>\n");
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Print out a tail in OSM XML format.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -633,6 +804,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "                  [--dump [--node=<node> ...]\n"
          "                          [--segment=<segment> ...]\n"
          "                          [--way=<way> ...]]\n"
+         "                          [--turn-relation=<rel> ...]]\n"
          "                  [--dump-osm [--no-super]\n"
          "                              [--latmin=<latmin> --latmax=<latmax>\n"
          "                               --lonmin=<lonmin> --lonmax=<lonmax>]]\n");
@@ -668,6 +840,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "      junctions = segment count at each junction.\n"
             "      super     = super-node and super-segments.\n"
             "      oneway    = oneway segments.\n"
+            "      turns     = turn restrictions.\n"
             "      speed     = speed limits.\n"
             "      weight    = weight limits.\n"
             "      height    = height limits.\n"
@@ -675,9 +848,10 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "      length    = length limits.\n"
             "\n"
             "--dump                    Dump selected contents of the database.\n"
-            "  --node=<node>           * the node with the selected number.\n"
-            "  --segment=<segment>     * the segment with the selected number.\n"
-            "  --way=<way>             * the way with the selected number.\n"
+            "  --node=<node>           * the node with the selected index.\n"
+            "  --segment=<segment>     * the segment with the selected index.\n"
+            "  --way=<way>             * the way with the selected index.\n"
+            "  --turn-relation=<rel>   * the turn relation with the selected index.\n"
             "                          Use 'all' instead of a number to get all of them.\n"
             "\n"
             "--dump-osm                Dump all or part of the database as an XML file.\n"
