@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/functionsx.h,v 1.6 2010/09/17 17:44:15 amb Exp $
-
- Header file for function prototypes for extended data types.
+ Header file for miscellaneous function prototypes for extended data types.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -28,10 +26,9 @@
 #include <stdio.h>
 
 #include "typesx.h"
-#include "profiles.h"
 
 
-/* In osmparser.c */
+/* Functions in osmparser.c */
 
 int ParseOSM(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations);
 
