@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/nodes.c,v 1.44 2010/07/26 18:17:20 amb Exp $
-
  Node data type functions.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -34,10 +32,15 @@
 #include "profiles.h"
 
 
+/* Local functions */
+
+static int valid_segment_for_profile(Ways *ways,Segment *segment,Profile *profile);
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Load in a node list from a file.
 
-  Nodes* LoadNodeList Returns the node list.
+  Nodes *LoadNodeList Returns the node list.
 
   const char *filename The name of the file to load.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -45,6 +48,10 @@
 Nodes *LoadNodeList(const char *filename)
 {
  Nodes *nodes;
+#if SLIM
+ size_t sizeoffsets;
+ int i;
+#endif
 
  nodes=(Nodes*)malloc(sizeof(Nodes));
 
@@ -69,11 +76,16 @@ Nodes *LoadNodeList(const char *filename)
 
  ReadFile(nodes->fd,&nodes->file,sizeof(NodesFile));
 
- nodes->nodesoffset=sizeof(NodesFile)+(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
+ sizeoffsets=(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
 
- nodes->incache[0]=NO_NODE;
- nodes->incache[1]=NO_NODE;
- nodes->incache[2]=NO_NODE;
+ nodes->offsets=(index_t*)malloc(sizeoffsets);
+
+ ReadFile(nodes->fd,nodes->offsets,sizeoffsets);
+
+ nodes->nodesoffset=sizeof(NodesFile)+sizeoffsets;
+
+ for(i=0;i<sizeof(nodes->cached)/sizeof(nodes->cached[0]);i++)
+    nodes->incache[i]=NO_NODE;
 
 #endif
 
@@ -82,11 +94,12 @@ Nodes *LoadNodeList(const char *filename)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the closest node given its latitude, longitude and optionally profile.
+  Find the closest node given its latitude, longitude and the profile of the
+  mode of transport that must be able to move to/from this node.
 
   index_t FindClosestNode Returns the closest node.
 
-  Nodes* nodes The set of nodes to search.
+  Nodes *nodes The set of nodes to search.
 
   Segments *segments The set of segments to use.
 
@@ -96,14 +109,14 @@ Nodes *LoadNodeList(const char *filename)
 
   double longitude The longitude to look for.
 
-  distance_t distance The maximum distance to look.
+  distance_t distance The maximum distance to look from the specified coordinates.
 
-  Profile *profile The profile of the mode of transport (or NULL).
+  Profile *profile The profile of the mode of transport.
 
   distance_t *bestdist Returns the distance to the best node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitude,double longitude,
+index_t FindClosestNode(Nodes *nodes,Segments *segments,Ways *ways,double latitude,double longitude,
                         distance_t distance,Profile *profile,distance_t *bestdist)
 {
  ll_bin_t   latbin=latlong_to_bin(radians_to_latlong(latitude ))-nodes->file.latzero;
@@ -117,7 +130,8 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
  do
    {
-    int latb,lonb,llbin;
+    ll_bin_t latb,lonb;
+    ll_bin2_t llbin;
 
     count=0;
 
@@ -180,7 +194,7 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
           for(i=index1;i<index2;i++)
             {
-             Node *node=LookupNode(nodes,i,1);
+             Node *node=LookupNode(nodes,i,3);
              double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
              double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
 
@@ -188,30 +202,25 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
              if(dist<distance)
                {
-                if(profile)
+                Segment *segment;
+
+                /* Check that at least one segment is valid for the profile */
+
+                segment=FirstSegment(segments,node,1);
+
+                do
                   {
-                   Segment *segment;
-
-                   /* Decide if this is node is valid for the profile */
-
-                   segment=FirstSegment(segments,nodes,i);
-
-                   do
+                   if(IsNormalSegment(segment) && valid_segment_for_profile(ways,segment,profile))
                      {
-                      Way *way=LookupWay(ways,segment->way,1);
+                      bestn=i;
+                      bestd=distance=dist;
 
-                      if(way->allow&profile->allow)
-                         break;
-
-                      segment=NextSegment(segments,segment,i);
+                      break;
                      }
-                   while(segment);
 
-                   if(!segment)
-                      continue;
+                   segment=NextSegment(segments,segment,i);
                   }
-
-                bestn=i; bestd=distance=dist;
+                while(segment);
                }
             }
 
@@ -230,13 +239,15 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the closest segment to a latitude, longitude and optionally profile.
+  Find the closest point on the closest segment given its latitude, longitude
+  and the profile of the mode of transport that must be able to move along this
+  segment.
 
   index_t FindClosestSegment Returns the closest segment index.
 
-  Nodes* nodes The set of nodes to search.
+  Nodes *nodes The set of nodes to use.
 
-  Segments *segments The set of segments to use.
+  Segments *segments The set of segments to search.
 
   Ways *ways The set of ways to use.
 
@@ -244,22 +255,22 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
   double longitude The longitude to look for.
 
-  distance_t distance The maximum distance to look.
+  distance_t distance The maximum distance to look from the specified coordinates.
 
-  Profile *profile The profile of the mode of transport (or NULL).
+  Profile *profile The profile of the mode of transport.
 
-  distance_t *bestdist Returns the distance to the best segment.
+  distance_t *bestdist Returns the distance to the closest point on the best segment.
 
-  index_t *bestnode1 Returns the best node at one end.
+  index_t *bestnode1 Returns the index of the node at one end of the closest segment.
 
-  index_t *bestnode2 Returns the best node at the other end.
+  index_t *bestnode2 Returns the index of the node at the other end of the closest segment.
 
-  distance_t *bestdist1 Returns the distance to the best node at one end.
+  distance_t *bestdist1 Returns the distance along the segment to the node at one end.
 
-  distance_t *bestdist2 Returns the distance to the best node at the other end.
+  distance_t *bestdist2 Returns the distance along the segment to the node at the other end.
   ++++++++++++++++++++++++++++++++++++++*/
 
-index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double latitude,double longitude,
+index_t FindClosestSegment(Nodes *nodes,Segments *segments,Ways *ways,double latitude,double longitude,
                            distance_t distance,Profile *profile, distance_t *bestdist,
                            index_t *bestnode1,index_t *bestnode2,distance_t *bestdist1,distance_t *bestdist2)
 {
@@ -275,7 +286,8 @@ index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double lat
 
  do
    {
-    int latb,lonb,llbin;
+    ll_bin_t latb,lonb;
+    ll_bin2_t llbin;
 
     count=0;
 
@@ -338,7 +350,7 @@ index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double lat
 
           for(i=index1;i<index2;i++)
             {
-             Node *node=LookupNode(nodes,i,1);
+             Node *node=LookupNode(nodes,i,3);
              double lat1=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
              double lon1=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
              distance_t dist1;
@@ -351,57 +363,65 @@ index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double lat
 
                 /* Check each segment for closeness and if valid for the profile */
 
-                segment=FirstSegment(segments,nodes,i);
+                segment=FirstSegment(segments,node,1);
 
                 do
                   {
-                   if(IsNormalSegment(segment))
+                   if(IsNormalSegment(segment) && valid_segment_for_profile(ways,segment,profile))
                      {
-                      Way *way=NULL;
+                      distance_t dist2,dist3;
+                      double lat2,lon2,dist3a,dist3b,distp;
 
-                      if(profile)
-                         way=LookupWay(ways,segment->way,1);
+                      GetLatLong(nodes,OtherNode(segment,i),&lat2,&lon2);
 
-                      if(!profile || way->allow&profile->allow)
+                      dist2=Distance(lat2,lon2,latitude,longitude);
+
+                      dist3=Distance(lat1,lon1,lat2,lon2);
+
+                      /* Use law of cosines (assume flat Earth) */
+
+                      dist3a=((double)dist1*(double)dist1-(double)dist2*(double)dist2+(double)dist3*(double)dist3)/(2.0*(double)dist3);
+                      dist3b=(double)dist3-dist3a;
+
+                      if((dist1+dist2)<dist3)
                         {
-                         distance_t dist2,dist3;
-                         double lat2,lon2,dist3a,dist3b,distp;
+                         distp=0;
+                        }
+                      else if(dist3a>=0 && dist3b>=0)
+                         distp=sqrt((double)dist1*(double)dist1-dist3a*dist3a);
+                      else if(dist3a>0)
+                        {
+                         distp=dist2;
+                         dist3a=dist3;
+                         dist3b=0;
+                        }
+                      else /* if(dist3b>0) */
+                        {
+                         distp=dist1;
+                         dist3a=0;
+                         dist3b=dist3;
+                        }
 
-                         GetLatLong(nodes,OtherNode(segment,i),&lat2,&lon2);
+                      if(distp<(double)bestd)
+                        {
+                         bests=IndexSegment(segments,segment);
 
-                         dist2=Distance(lat2,lon2,latitude,longitude);
-
-                         dist3=Distance(lat1,lon1,lat2,lon2);
-
-                         /* Use law of cosines (assume flat Earth) */
-
-                         dist3a=((double)dist1*(double)dist1-(double)dist2*(double)dist2+(double)dist3*(double)dist3)/(2.0*(double)dist3);
-                         dist3b=(double)dist3-dist3a;
-
-                         if(dist3a>=0 && dist3b>=0)
-                            distp=sqrt((double)dist1*(double)dist1-dist3a*dist3a);
-                         else if(dist3a>0)
+                         if(segment->node1==i)
                            {
-                            distp=dist2;
-                            dist3a=dist3;
-                            dist3b=0;
-                           }
-                         else /* if(dist3b>0) */
-                           {
-                            distp=dist1;
-                            dist3a=0;
-                            dist3b=dist3;
-                           }
-
-                         if(distp<(double)bestd)
-                           {
-                            bests=IndexSegment(segments,segment);
                             bestn1=i;
                             bestn2=OtherNode(segment,i);
                             bestd1=(distance_t)dist3a;
                             bestd2=(distance_t)dist3b;
-                            bestd=(distance_t)distp;
                            }
+                         else
+                           {
+                            bestn1=OtherNode(segment,i);
+                            bestn2=i;
+                            bestd1=(distance_t)dist3b;
+                            bestd2=(distance_t)dist3a;
+                           }
+
+                         bestd=(distance_t)distp;
                         }
                      }
 
@@ -431,10 +451,62 @@ index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double lat
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the transport defined by the profile is allowed on the segment.
+
+  int valid_segment_for_profile Return 1 if it is or 0 if not.
+
+  Ways *ways The set of ways to use.
+
+  Segment *segment The segment to check.
+
+  Profile *profile The profile to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int valid_segment_for_profile(Ways *ways,Segment *segment,Profile *profile)
+{
+ Way *way=LookupWay(ways,segment->way,1);
+ score_t segment_pref;
+ int i;
+
+ /* mode of transport must be allowed on the highway */
+ if(!(way->allow&profile->allow))
+    return(0);
+
+ /* must obey weight restriction (if exists) */
+ if(way->weight && way->weight<profile->weight)
+    return(0);
+
+ /* must obey height/width/length restriction (if exists) */
+ if((way->height && way->height<profile->height) ||
+    (way->width  && way->width <profile->width ) ||
+    (way->length && way->length<profile->length))
+    return(0);
+
+ segment_pref=profile->highway[HIGHWAY(way->type)];
+
+ for(i=1;i<Property_Count;i++)
+    if(ways->file.props & PROPERTIES(i))
+      {
+       if(way->props & PROPERTIES(i))
+          segment_pref*=profile->props_yes[i];
+       else
+          segment_pref*=profile->props_no[i];
+      }
+
+ /* profile preferences must allow this highway */
+ if(segment_pref==0)
+    return(0);
+
+ /* Must be OK */
+ return(1);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Get the latitude and longitude associated with a node.
 
-  Nodes *nodes The set of nodes.
+  Nodes *nodes The set of nodes to use.
 
   index_t index The node index.
 
@@ -445,17 +517,17 @@ index_t FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double lat
 
 void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
 {
- Node *node=LookupNode(nodes,index,2);
- int latbin=-1,lonbin=-1;
- int start,end,mid;
+ Node *node=LookupNode(nodes,index,4);
+ ll_bin_t latbin=-1,lonbin=-1;
+ ll_bin_t start,end,mid;
  index_t offset;
 
- /* Binary search - search key closest below is required.
+ /* Binary search - search key nearest match below is required.
   *
   *  # <- start  |  Check mid and move start or end if it doesn't match
   *  #           |
-  *  #           |  Since an inexact match is wanted we must set end=mid-1
-  *  # <- mid    |  or start=mid because we know that mid doesn't match.
+  *  #           |  A lower bound match is wanted we can set end=mid-1 or
+  *  # <- mid    |  start=mid because we know that mid doesn't match.
   *  #           |
   *  #           |  Eventually either end=start or end=start+1 and one of
   *  # <- end    |  start or end is the wanted one.
@@ -472,10 +544,10 @@ void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
 
     offset=LookupNodeOffset(nodes,nodes->file.latbins*mid);
 
-    if(offset<index)                    /* Mid point is too low */
+    if(offset<index)                    /* Mid point is too low for an exact match but could be lower bound */
        start=mid;
     else if(offset>index)               /* Mid point is too high */
-       end=mid-1;
+       end=mid?(mid-1):mid;
     else                                /* Mid point is correct */
       {lonbin=mid;break;}
    }
@@ -506,10 +578,10 @@ void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
 
     offset=LookupNodeOffset(nodes,lonbin*nodes->file.latbins+mid);
 
-    if(offset<index)                    /* Mid point is too low */
+    if(offset<index)                    /* Mid point is too low for an exact match but could be lower bound */
        start=mid;
     else if(offset>index)               /* Mid point is too high */
-       end=mid-1;
+       end=mid?(mid-1):mid;
     else                                /* Mid point is correct */
       {latbin=mid;break;}
    }
