@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/translations.c,v 1.13 2010/09/15 18:30:08 amb Exp $
-
  Load the translations from a file and the functions for handling them.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010 Andrew M. Bishop
+ This file Copyright 2010-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -33,17 +31,21 @@
 
 /* Global variables - default English values - Must not require any UTF-8 encoding */
 
-char *translate_copyright_creator[2]={"Creator","Routino - http://www.routino.org/"};
-char *translate_copyright_source[2] ={NULL,NULL};
-char *translate_copyright_license[2]={NULL,NULL};
+char *translate_raw_copyright_creator[2]={"Creator","Routino - http://www.routino.org/"};
+char *translate_raw_copyright_source[2] ={NULL,NULL};
+char *translate_raw_copyright_license[2]={NULL,NULL};
 
-char *translate_heading[9]={"South","South-West","West","North-West","North","North-East","East","South-East","South"};
-char *translate_turn[9]   ={"Very sharp left","Sharp left","Left","Slight left","Straight on","Slight right","Right","Sharp right","Very sharp right"};
+char *translate_xml_copyright_creator[2]={"Creator","Routino - http://www.routino.org/"};
+char *translate_xml_copyright_source[2] ={NULL,NULL};
+char *translate_xml_copyright_license[2]={NULL,NULL};
 
-char *translate_highway[Way_Count]={"","motorway","trunk road","primary road","secondary road","tertiary road","unclassified road","residential road","service road","track","cycleway","path","steps","ferry"};
+char *translate_xml_heading[9]={"South","South-West","West","North-West","North","North-East","East","South-East","South"};
+char *translate_xml_turn[9]   ={"Very sharp left","Sharp left","Left","Slight left","Straight on","Slight right","Right","Sharp right","Very sharp right"};
 
-char *translate_route_shortest="Shortest";
-char *translate_route_quickest="Quickest";
+char *translate_raw_highway[Way_Count]={"","motorway","trunk road","primary road","secondary road","tertiary road","unclassified road","residential road","service road","track","cycleway","path","steps","ferry"};
+
+char *translate_xml_route_shortest="Shortest";
+char *translate_xml_route_quickest="Quickest";
 
 char *translate_html_waypoint="<span class='w'>Waypoint</span>";
 char *translate_html_junction="Junction";
@@ -315,11 +317,14 @@ static int CopyrightCreatorType_function(const char *_tag_,int _type_,const char
     XMLPARSE_ASSERT_STRING(_tag_,string);
     XMLPARSE_ASSERT_STRING(_tag_,text);
 
+    translate_raw_copyright_creator[0]=strcpy(malloc(strlen(string)+1),string);
+    translate_raw_copyright_creator[1]=strcpy(malloc(strlen(text)+1)  ,text);
+
     xmlstring=ParseXML_Encode_Safe_XML(string);
     xmltext  =ParseXML_Encode_Safe_XML(text);
 
-    translate_copyright_creator[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
-    translate_copyright_creator[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
+    translate_xml_copyright_creator[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_xml_copyright_creator[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
    }
 
  return(0);
@@ -349,11 +354,14 @@ static int CopyrightSourceType_function(const char *_tag_,int _type_,const char
     XMLPARSE_ASSERT_STRING(_tag_,string);
     XMLPARSE_ASSERT_STRING(_tag_,text);
 
+    translate_raw_copyright_source[0]=strcpy(malloc(strlen(string)+1),string);
+    translate_raw_copyright_source[1]=strcpy(malloc(strlen(text)+1)  ,text);
+
     xmlstring=ParseXML_Encode_Safe_XML(string);
     xmltext  =ParseXML_Encode_Safe_XML(text);
 
-    translate_copyright_source[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
-    translate_copyright_source[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
+    translate_xml_copyright_source[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_xml_copyright_source[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
    }
 
  return(0);
@@ -383,11 +391,14 @@ static int CopyrightLicenseType_function(const char *_tag_,int _type_,const char
     XMLPARSE_ASSERT_STRING(_tag_,string);
     XMLPARSE_ASSERT_STRING(_tag_,text);
 
+    translate_raw_copyright_license[0]=strcpy(malloc(strlen(string)+1),string);
+    translate_raw_copyright_license[1]=strcpy(malloc(strlen(text)+1)  ,text);
+
     xmlstring=ParseXML_Encode_Safe_XML(string);
     xmltext  =ParseXML_Encode_Safe_XML(text);
 
-    translate_copyright_license[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
-    translate_copyright_license[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
+    translate_xml_copyright_license[0]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_xml_copyright_license[1]=strcpy(malloc(strlen(xmltext)+1)  ,xmltext);
    }
 
  return(0);
@@ -413,10 +424,9 @@ static int TurnType_function(const char *_tag_,int _type_,const char *direction,
  if(_type_&XMLPARSE_TAG_START && store)
    {
     char *xmlstring;
-
     int d;
 
-    XMLPARSE_ASSERT_INTEGER(_tag_,direction,d);
+    XMLPARSE_ASSERT_INTEGER(_tag_,direction); d=atoi(direction);
     XMLPARSE_ASSERT_STRING(_tag_,string);
 
     d+=4;
@@ -426,7 +436,7 @@ static int TurnType_function(const char *_tag_,int _type_,const char *direction,
 
     xmlstring=ParseXML_Encode_Safe_XML(string);
 
-    translate_turn[d]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_xml_turn[d]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
    }
 
  return(0);
@@ -454,7 +464,7 @@ static int HeadingType_function(const char *_tag_,int _type_,const char *directi
     char *xmlstring;
     int d;
 
-    XMLPARSE_ASSERT_INTEGER(_tag_,direction,d);
+    XMLPARSE_ASSERT_INTEGER(_tag_,direction); d=atoi(direction);
     XMLPARSE_ASSERT_STRING(_tag_,string);
 
     d+=4;
@@ -464,7 +474,7 @@ static int HeadingType_function(const char *_tag_,int _type_,const char *directi
 
     xmlstring=ParseXML_Encode_Safe_XML(string);
 
-    translate_heading[d]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_xml_heading[d]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
    }
 
  return(0);
@@ -489,7 +499,6 @@ static int HighwayType_function(const char *_tag_,int _type_,const char *type,co
 {
  if(_type_&XMLPARSE_TAG_START && store)
    {
-    char *xmlstring;
     Highway highway;
 
     XMLPARSE_ASSERT_STRING(_tag_,type);
@@ -500,9 +509,7 @@ static int HighwayType_function(const char *_tag_,int _type_,const char *type,co
     if(highway==Way_Count)
        XMLPARSE_INVALID(_tag_,type);
 
-    xmlstring=ParseXML_Encode_Safe_XML(string);
-
-    translate_highway[highway]=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+    translate_raw_highway[highway]=strcpy(malloc(strlen(string)+1),string);
    }
 
  return(0);
@@ -535,9 +542,9 @@ static int RouteType_function(const char *_tag_,int _type_,const char *type,cons
     xmlstring=ParseXML_Encode_Safe_XML(string);
 
     if(!strcmp(type,"shortest"))
-       translate_route_shortest=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+       translate_xml_route_shortest=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
     else if(!strcmp(type,"quickest"))
-       translate_route_quickest=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
+       translate_xml_route_quickest=strcpy(malloc(strlen(xmlstring)+1),xmlstring);
     else
        XMLPARSE_INVALID(_tag_,type);
    }
@@ -1099,6 +1106,7 @@ static int languageType_function(const char *_tag_,int _type_,const char *lang)
 
 int ParseXMLTranslations(const char *filename,const char *language)
 {
+ FILE *file;
  int retval;
 
  store_lang=language;
@@ -1109,7 +1117,7 @@ int ParseXMLTranslations(const char *filename,const char *language)
     return(1);
    }
 
- FILE *file=fopen(filename,"r");
+ file=fopen(filename,"r");
 
  if(!file)
    {
